'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var coreClient = require('@azure-rest/core-client');
var corePaging = require('@azure/core-paging');

// Copyright (c) Microsoft Corporation.
/**
 * Initialize a new instance of the class AnomalyDetectorRestClient class.
 * @param Endpoint type: string
 * @param credentials type: KeyCredential
 */
function createClient(Endpoint, credentials, options = {}) {
    var _a, _b;
    const ApiVersion = (_a = options.ApiVersion) !== null && _a !== void 0 ? _a : "v1.1";
    const baseUrl = (_b = options.baseUrl) !== null && _b !== void 0 ? _b : `${Endpoint}/anomalydetector/${ApiVersion}`;
    options = Object.assign(Object.assign({}, options), { credentials: {
            apiKeyHeaderName: "Ocp-Apim-Subscription-Key",
        } });
    const userAgentInfo = `azsdk-js-ai-anomaly-detector-rest/1.0.0-beta.1`;
    const userAgentPrefix = options.userAgentOptions && options.userAgentOptions.userAgentPrefix
        ? `${options.userAgentOptions.userAgentPrefix} ${userAgentInfo}`
        : `${userAgentInfo}`;
    options = Object.assign(Object.assign({}, options), { userAgentOptions: {
            userAgentPrefix,
        } });
    const client = coreClient.getClient(baseUrl, credentials, options);
    return client;
}

// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
const responseMap = {
    "POST /timeseries/entire/detect": ["200"],
    "POST /timeseries/last/detect": ["200"],
    "POST /timeseries/changepoint/detect": ["200"],
    "GET /multivariate/detect-batch/{resultId}": ["200"],
    "POST /multivariate/models": ["201"],
    "GET /multivariate/models": ["200"],
    "DELETE /multivariate/models/{modelId}": ["204"],
    "GET /multivariate/models/{modelId}": ["200"],
    "POST /multivariate/models/{modelId}:detect-batch": ["202"],
    "POST /multivariate/models/{modelId}:detect-last": ["200"],
};
function isUnexpected(response) {
    const lroOriginal = response.headers["x-ms-original-url"];
    const url = new URL(lroOriginal !== null && lroOriginal !== void 0 ? lroOriginal : response.request.url);
    const method = response.request.method;
    let pathDetails = responseMap[`${method} ${url.pathname}`];
    if (!pathDetails) {
        pathDetails = getParametrizedPathSuccess(method, url.pathname);
    }
    return !pathDetails.includes(response.status);
}
function getParametrizedPathSuccess(method, path) {
    var _a, _b, _c, _d;
    const pathParts = path.split("/");
    // Traverse list to match the longest candidate
    // matchedLen: the length of candidate path
    // matchedValue: the matched status code array
    let matchedLen = -1, matchedValue = [];
    // Iterate the responseMap to find a match
    for (const [key, value] of Object.entries(responseMap)) {
        // Extracting the path from the map key which is in format
        // GET /path/foo
        if (!key.startsWith(method)) {
            continue;
        }
        const candidatePath = getPathFromMapKey(key);
        // Get each part of the url path
        const candidateParts = candidatePath.split("/");
        // track if we have found a match to return the values found.
        let found = true;
        for (let i = candidateParts.length - 1, j = pathParts.length - 1; i >= 1 && j >= 1; i--, j--) {
            if (((_a = candidateParts[i]) === null || _a === void 0 ? void 0 : _a.startsWith("{")) && ((_b = candidateParts[i]) === null || _b === void 0 ? void 0 : _b.indexOf("}")) !== -1) {
                const start = candidateParts[i].indexOf("}") + 1, end = (_c = candidateParts[i]) === null || _c === void 0 ? void 0 : _c.length;
                // If the current part of the candidate is a "template" part
                // Try to use the suffix of pattern to match the path
                // {guid} ==> $
                // {guid}:export ==> :export$
                const isMatched = new RegExp(`${(_d = candidateParts[i]) === null || _d === void 0 ? void 0 : _d.slice(start, end)}`).test(pathParts[j] || "");
                if (!isMatched) {
                    found = false;
                    break;
                }
                continue;
            }
            // If the candidate part is not a template and
            // the parts don't match mark the candidate as not found
            // to move on with the next candidate path.
            if (candidateParts[i] !== pathParts[j]) {
                found = false;
                break;
            }
        }
        // We finished evaluating the current candidate parts
        // Update the matched value if and only if we found the longer pattern
        if (found && candidatePath.length > matchedLen) {
            matchedLen = candidatePath.length;
            matchedValue = value;
        }
    }
    return matchedValue;
}
function getPathFromMapKey(mapKey) {
    const pathStart = mapKey.indexOf("/");
    return mapKey.slice(pathStart);
}

// Copyright (c) Microsoft Corporation.
/**
 * Helper to paginate results from an initial response that follows the specification of Autorest `x-ms-pageable` extension
 * @param client - Client to use for sending the next page requests
 * @param initialResponse - Initial response containing the nextLink and current page of elements
 * @param customGetPage - Optional - Function to define how to extract the page and next link to be used to paginate the results
 * @returns - PagedAsyncIterableIterator to iterate the elements
 */
function paginate(client, initialResponse, options = {}) {
    let firstRun = true;
    // We need to check the response for success before trying to inspect it looking for
    // the properties to use for nextLink and itemName
    checkPagingRequest(initialResponse);
    const { itemName, nextLinkName } = getPaginationProperties(initialResponse);
    const { customGetPage } = options;
    const pagedResult = {
        firstPageLink: "",
        getPage: typeof customGetPage === "function"
            ? customGetPage
            : async (pageLink) => {
                const result = firstRun ? initialResponse : await client.pathUnchecked(pageLink).get();
                firstRun = false;
                checkPagingRequest(result);
                const nextLink = getNextLink(result.body, nextLinkName);
                const values = getElements(result.body, itemName);
                return {
                    page: values,
                    nextPageLink: nextLink,
                };
            },
    };
    return corePaging.getPagedAsyncIterator(pagedResult);
}
/**
 * Gets for the value of nextLink in the body
 */
function getNextLink(body, nextLinkName) {
    if (!nextLinkName) {
        return undefined;
    }
    const nextLink = body[nextLinkName];
    if (typeof nextLink !== "string" && typeof nextLink !== "undefined") {
        throw new Error(`Body Property ${nextLinkName} should be a string or undefined`);
    }
    return nextLink;
}
/**
 * Gets the elements of the current request in the body.
 */
function getElements(body, itemName) {
    const value = body[itemName];
    // value has to be an array according to the x-ms-pageable extension.
    // The fact that this must be an array is used above to calculate the
    // type of elements in the page in PaginateReturn
    if (!Array.isArray(value)) {
        throw new Error(`Couldn't paginate response\n Body doesn't contain an array property with name: ${itemName}`);
    }
    return value !== null && value !== void 0 ? value : [];
}
/**
 * Checks if a request failed
 */
function checkPagingRequest(response) {
    const Http2xxStatusCodes = ["200", "201", "202", "203", "204", "205", "206", "207", "208", "226"];
    if (!Http2xxStatusCodes.includes(response.status)) {
        throw coreClient.createRestError(`Pagination failed with unexpected statusCode ${response.status}`, response);
    }
}
/**
 * Extracts the itemName and nextLinkName from the initial response to use them for pagination
 */
function getPaginationProperties(initialResponse) {
    // Build a set with the passed custom nextLinkNames
    const nextLinkNames = new Set(["nextLink"]);
    // Build a set with the passed custom set of itemNames
    const itemNames = new Set(["value", "models"]);
    let nextLinkName;
    let itemName;
    for (const name of nextLinkNames) {
        const nextLink = initialResponse.body[name];
        if (nextLink) {
            nextLinkName = name;
            break;
        }
    }
    for (const name of itemNames) {
        const item = initialResponse.body[name];
        if (item) {
            itemName = name;
            break;
        }
    }
    if (!itemName) {
        throw new Error(`Couldn't paginate response\n Body doesn't contain an array property with name: ${[
            ...itemNames,
        ].join(" OR ")}`);
    }
    return { itemName, nextLinkName };
}

// Copyright (c) Microsoft Corporation.

exports["default"] = createClient;
exports.isUnexpected = isUnexpected;
exports.paginate = paginate;
//# sourceMappingURL=index.js.map
