import { Client } from '@azure-rest/core-client';
import { ClientOptions } from '@azure-rest/core-client';
import { HttpResponse } from '@azure-rest/core-client';
import { KeyCredential } from '@azure/core-auth';
import { PagedAsyncIterableIterator } from '@azure/core-paging';
import { PathUncheckedResponse } from '@azure-rest/core-client';
import { RawHttpHeaders } from '@azure/core-rest-pipeline';
import { RequestParameters } from '@azure-rest/core-client';
import { StreamableMethod } from '@azure-rest/core-client';

/** An optional field, indicating the manner to align multiple variables. */
export declare interface AlignPolicy {
    /**
     * An optional field, indicating how to align different variables to the same
     * time-range. Either Inner or Outer.
     */
    alignMode?: "Inner" | "Outer";
    /**
     * An optional field, indicating how missing values will be filled. One of
     * Previous, Subsequent, Linear, Zero, Fixed.
     *
     * Possible values: Previous, Subsequent, Linear, Zero, Fixed
     */
    fillNAMethod?: string;
    /** An optional field. Required when fillNAMethod is Fixed. */
    paddingValue?: number;
}

/** An optional field, indicating the manner to align multiple variables. */
export declare interface AlignPolicyOutput {
    /**
     * An optional field, indicating how to align different variables to the same
     * time-range. Either Inner or Outer.
     */
    alignMode?: "Inner" | "Outer";
    /**
     * An optional field, indicating how missing values will be filled. One of
     * Previous, Subsequent, Linear, Zero, Fixed.
     *
     * Possible values: Previous, Subsequent, Linear, Zero, Fixed
     */
    fillNAMethod?: string;
    /** An optional field. Required when fillNAMethod is Fixed. */
    paddingValue?: number;
}

/** Response of getting a model. */
export declare interface AnomalyDetectionModelOutput {
    /** Model identifier. */
    modelId: string;
    /** Date and time (UTC) when the model was created. */
    createdTime: string;
    /** Date and time (UTC) when the model was last updated. */
    lastUpdatedTime: string;
    /**
     * Training result of a model including its status, errors and diagnostics
     * information.
     */
    modelInfo?: ModelInfoOutput;
}

/** Error information returned by the API. */
export declare interface AnomalyDetectorErrorOutput {
    /**
     * The error code.
     *
     * Possible values: InvalidCustomInterval, BadArgument, InvalidGranularity, InvalidPeriod, InvalidModelArgument, InvalidSeries, InvalidJsonFormat, RequiredGranularity, RequiredSeries, InvalidImputeMode, InvalidImputeFixedValue
     */
    code?: string;
    /** A message explaining the error reported by the service. */
    message?: string;
}

export declare type AnomalyDetectorRestClient = Client & {
    path: Routes;
};

export declare interface AnomalyDetectorRestClientOptions extends ClientOptions {
    ApiVersion?: string;
}

/** Interpretation of the anomalous timestamp. */
export declare interface AnomalyInterpretationOutput {
    /** Variable. */
    variable?: string;
    /**
     * This score shows the percentage contributing to the anomalous timestamp. A
     * number between 0 and 1.
     */
    contributionScore?: number;
    /** Correlation changes among the anomalous variables */
    correlationChanges?: CorrelationChangesOutput;
}

/** Anomaly status and information. */
export declare interface AnomalyStateOutput {
    /** The timestamp for this anomaly. */
    timestamp: string;
    /** The detailed value of this anomalous timestamp. */
    value?: AnomalyValueOutput;
    /** Error message for the current timestamp. */
    errors?: Array<ErrorResponseOutput>;
}

/** Detailed information of the anomalous timestamp. */
export declare interface AnomalyValueOutput {
    /** True if an anomaly is detected at the current timestamp. */
    isAnomaly: boolean;
    /**
     * Indicates the significance of the anomaly. The higher the severity, the more
     * significant the anomaly is.
     */
    severity: number;
    /**
     * Raw anomaly score of severity, will help indicate the degree of abnormality as
     * well.
     */
    score: number;
    /** Interpretation of this anomalous timestamp. */
    interpretation?: Array<AnomalyInterpretationOutput>;
}

/** Correlation changes among the anomalous variables */
export declare interface CorrelationChangesOutput {
    /** The correlated variables that have correlation changes under an anomaly. */
    changedVariables?: string[];
}

/**
 * Initialize a new instance of the class AnomalyDetectorRestClient class.
 * @param Endpoint type: string
 * @param credentials type: KeyCredential
 */
declare function createClient(Endpoint: string, credentials: KeyCredential, options?: AnomalyDetectorRestClientOptions): AnomalyDetectorRestClient;
export default createClient;

export declare interface DeleteMultivariateModel {
    /** Delete an existing multivariate model according to the modelId */
    delete(options?: DeleteMultivariateModelParameters): StreamableMethod<DeleteMultivariateModel204Response | DeleteMultivariateModelDefaultResponse>;
    /**
     * Get detailed information of multivariate model, including the training status
     * and variables used in the model.
     */
    get(options?: GetMultivariateModelParameters): StreamableMethod<GetMultivariateModel200Response | GetMultivariateModelDefaultResponse>;
}

/** There is no content to send for this request, but the headers may be useful. */
export declare interface DeleteMultivariateModel204Response extends HttpResponse {
    status: "204";
}

export declare interface DeleteMultivariateModelDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface DeleteMultivariateModelDefaultResponse extends HttpResponse {
    status: string;
    body: ErrorResponseOutput;
    headers: RawHttpHeaders & DeleteMultivariateModelDefaultHeaders;
}

export declare type DeleteMultivariateModelParameters = RequestParameters;

export declare interface DetectMultivariateBatchAnomaly {
    /**
     * Submit multivariate anomaly detection task with the modelId of trained model
     * and inference data, the input schema should be the same with the training
     * request. The request will complete asynchronously and return a resultId to
     * query the detection result.The request should be a source link to indicate an
     * externally accessible Azure storage Uri, either pointed to an Azure blob
     * storage folder, or pointed to a CSV file in Azure blob storage.
     */
    post(options: DetectMultivariateBatchAnomalyParameters): StreamableMethod<DetectMultivariateBatchAnomaly202Response | DetectMultivariateBatchAnomalyDefaultResponse>;
}

export declare interface DetectMultivariateBatchAnomaly202Headers {
    /** Id of the detection result. */
    "operation-id": string;
    /** Location of the detection result. */
    "operation-location": string;
}

/** The request has been accepted for processing, but processing has not yet completed. */
export declare interface DetectMultivariateBatchAnomaly202Response extends HttpResponse {
    status: "202";
    body: MultivariateDetectionResultOutput;
    headers: RawHttpHeaders & DetectMultivariateBatchAnomaly202Headers;
}

export declare interface DetectMultivariateBatchAnomalyBodyParam {
    /** Request of multivariate anomaly detection. */
    body: MultivariateBatchDetectionOptions;
}

export declare interface DetectMultivariateBatchAnomalyDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface DetectMultivariateBatchAnomalyDefaultResponse extends HttpResponse {
    status: string;
    body: ErrorResponseOutput;
    headers: RawHttpHeaders & DetectMultivariateBatchAnomalyDefaultHeaders;
}

export declare type DetectMultivariateBatchAnomalyParameters = DetectMultivariateBatchAnomalyBodyParam & RequestParameters;

export declare interface DetectMultivariateLastAnomaly {
    /**
     * Submit multivariate anomaly detection task with the modelId of trained model
     * and inference data, and the inference data should be put into request body in a
     * JSON format. The request will complete synchronously and return the detection
     * immediately in the response body.
     */
    post(options: DetectMultivariateLastAnomalyParameters): StreamableMethod<DetectMultivariateLastAnomaly200Response | DetectMultivariateLastAnomalyDefaultResponse>;
}

/** The request has succeeded. */
export declare interface DetectMultivariateLastAnomaly200Response extends HttpResponse {
    status: "200";
    body: MultivariateLastDetectionResultOutput;
}

export declare interface DetectMultivariateLastAnomalyBodyParam {
    /** Request of last detection. */
    body: MultivariateLastDetectionOptions;
}

export declare interface DetectMultivariateLastAnomalyDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface DetectMultivariateLastAnomalyDefaultResponse extends HttpResponse {
    status: string;
    body: ErrorResponseOutput;
    headers: RawHttpHeaders & DetectMultivariateLastAnomalyDefaultHeaders;
}

export declare type DetectMultivariateLastAnomalyParameters = DetectMultivariateLastAnomalyBodyParam & RequestParameters;

export declare interface DetectUnivariateChangePoint {
    /** Evaluate change point score of every series point */
    post(options: DetectUnivariateChangePointParameters): StreamableMethod<DetectUnivariateChangePoint200Response | DetectUnivariateChangePointDefaultResponse>;
}

/** The request has succeeded. */
export declare interface DetectUnivariateChangePoint200Response extends HttpResponse {
    status: "200";
    body: UnivariateChangePointDetectionResultOutput;
}

export declare interface DetectUnivariateChangePointBodyParam {
    /** Method of univariate anomaly detection. */
    body: UnivariateChangePointDetectionOptions;
}

export declare interface DetectUnivariateChangePointDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface DetectUnivariateChangePointDefaultResponse extends HttpResponse {
    status: string;
    body: AnomalyDetectorErrorOutput;
    headers: RawHttpHeaders & DetectUnivariateChangePointDefaultHeaders;
}

export declare type DetectUnivariateChangePointParameters = DetectUnivariateChangePointBodyParam & RequestParameters;

export declare interface DetectUnivariateEntireSeries {
    /**
     * This operation generates a model with an entire series, each point is detected
     * with the same model. With this method, points before and after a certain point
     * are used to determine whether it is an anomaly. The entire detection can give
     * user an overall status of the time series.
     */
    post(options: DetectUnivariateEntireSeriesParameters): StreamableMethod<DetectUnivariateEntireSeries200Response | DetectUnivariateEntireSeriesDefaultResponse>;
}

/** The request has succeeded. */
export declare interface DetectUnivariateEntireSeries200Response extends HttpResponse {
    status: "200";
    body: UnivariateEntireDetectionResultOutput;
}

export declare interface DetectUnivariateEntireSeriesBodyParam {
    /** Method of univariate anomaly detection. */
    body: UnivariateDetectionOptions;
}

export declare interface DetectUnivariateEntireSeriesDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface DetectUnivariateEntireSeriesDefaultResponse extends HttpResponse {
    status: string;
    body: AnomalyDetectorErrorOutput;
    headers: RawHttpHeaders & DetectUnivariateEntireSeriesDefaultHeaders;
}

export declare type DetectUnivariateEntireSeriesParameters = DetectUnivariateEntireSeriesBodyParam & RequestParameters;

export declare interface DetectUnivariateLastPoint {
    /**
     * This operation generates a model using the points that you sent into the API,
     * and based on all data to determine whether the last point is anomalous.
     */
    post(options: DetectUnivariateLastPointParameters): StreamableMethod<DetectUnivariateLastPoint200Response | DetectUnivariateLastPointDefaultResponse>;
}

/** The request has succeeded. */
export declare interface DetectUnivariateLastPoint200Response extends HttpResponse {
    status: "200";
    body: UnivariateLastDetectionResultOutput;
}

export declare interface DetectUnivariateLastPointBodyParam {
    /** Method of univariate anomaly detection. */
    body: UnivariateDetectionOptions;
}

export declare interface DetectUnivariateLastPointDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface DetectUnivariateLastPointDefaultResponse extends HttpResponse {
    status: string;
    body: AnomalyDetectorErrorOutput;
    headers: RawHttpHeaders & DetectUnivariateLastPointDefaultHeaders;
}

export declare type DetectUnivariateLastPointParameters = DetectUnivariateLastPointBodyParam & RequestParameters;

/** Diagnostics information to help inspect the states of model or variable. */
export declare interface DiagnosticsInfo {
    /** Model status. */
    modelState?: ModelState;
    /** Variable Status. */
    variableStates?: Array<VariableState>;
}

/** Diagnostics information to help inspect the states of model or variable. */
export declare interface DiagnosticsInfoOutput {
    /** Model status. */
    modelState?: ModelStateOutput;
    /** Variable Status. */
    variableStates?: Array<VariableStateOutput>;
}

/** ErrorResponse contains code and message that shows the error information. */
export declare interface ErrorResponse {
    /** The error code. */
    code: string;
    /** The message explaining the error reported by the service. */
    message: string;
}

/** ErrorResponse contains code and message that shows the error information. */
export declare interface ErrorResponseOutput {
    /** The error code. */
    code: string;
    /** The message explaining the error reported by the service. */
    message: string;
}

/**
 * Helper type to extract the type of an array
 */
export declare type GetArrayType<T> = T extends Array<infer TData> ? TData : never;

export declare interface GetMultivariateBatchDetectionResult {
    /**
     * For asynchronous inference, get multivariate anomaly detection result based on
     * resultId returned by the BatchDetectAnomaly api.
     */
    get(options?: GetMultivariateBatchDetectionResultParameters): StreamableMethod<GetMultivariateBatchDetectionResult200Response | GetMultivariateBatchDetectionResultDefaultResponse>;
}

/** The request has succeeded. */
export declare interface GetMultivariateBatchDetectionResult200Response extends HttpResponse {
    status: "200";
    body: MultivariateDetectionResultOutput;
}

export declare interface GetMultivariateBatchDetectionResultDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface GetMultivariateBatchDetectionResultDefaultResponse extends HttpResponse {
    status: string;
    body: ErrorResponseOutput;
    headers: RawHttpHeaders & GetMultivariateBatchDetectionResultDefaultHeaders;
}

export declare type GetMultivariateBatchDetectionResultParameters = RequestParameters;

/** The request has succeeded. */
export declare interface GetMultivariateModel200Response extends HttpResponse {
    status: "200";
    body: AnomalyDetectionModelOutput;
}

export declare interface GetMultivariateModelDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface GetMultivariateModelDefaultResponse extends HttpResponse {
    status: string;
    body: ErrorResponseOutput;
    headers: RawHttpHeaders & GetMultivariateModelDefaultHeaders;
}

export declare type GetMultivariateModelParameters = RequestParameters;

/**
 * The type of a custom function that defines how to get a page and a link to the next one if any.
 */
export declare type GetPage<TPage> = (pageLink: string, maxPageSize?: number) => Promise<{
    page: TPage;
    nextPageLink?: string;
}>;

export declare function isUnexpected(response: DetectUnivariateEntireSeries200Response | DetectUnivariateEntireSeriesDefaultResponse): response is DetectUnivariateEntireSeriesDefaultResponse;

export declare function isUnexpected(response: DetectUnivariateLastPoint200Response | DetectUnivariateLastPointDefaultResponse): response is DetectUnivariateLastPointDefaultResponse;

export declare function isUnexpected(response: DetectUnivariateChangePoint200Response | DetectUnivariateChangePointDefaultResponse): response is DetectUnivariateChangePointDefaultResponse;

export declare function isUnexpected(response: GetMultivariateBatchDetectionResult200Response | GetMultivariateBatchDetectionResultDefaultResponse): response is GetMultivariateBatchDetectionResultDefaultResponse;

export declare function isUnexpected(response: TrainMultivariateModel201Response | TrainMultivariateModelDefaultResponse): response is TrainMultivariateModelDefaultResponse;

export declare function isUnexpected(response: ListMultivariateModels200Response | ListMultivariateModelsDefaultResponse): response is ListMultivariateModelsDefaultResponse;

export declare function isUnexpected(response: DeleteMultivariateModel204Response | DeleteMultivariateModelDefaultResponse): response is DeleteMultivariateModelDefaultResponse;

export declare function isUnexpected(response: GetMultivariateModel200Response | GetMultivariateModelDefaultResponse): response is GetMultivariateModelDefaultResponse;

export declare function isUnexpected(response: DetectMultivariateBatchAnomaly202Response | DetectMultivariateBatchAnomalyDefaultResponse): response is DetectMultivariateBatchAnomalyDefaultResponse;

export declare function isUnexpected(response: DetectMultivariateLastAnomaly200Response | DetectMultivariateLastAnomalyDefaultResponse): response is DetectMultivariateLastAnomalyDefaultResponse;

/** The request has succeeded. */
export declare interface ListMultivariateModels200Response extends HttpResponse {
    status: "200";
    body: ModelListOutput;
}

export declare interface ListMultivariateModelsDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface ListMultivariateModelsDefaultResponse extends HttpResponse {
    status: string;
    body: ErrorResponseOutput;
    headers: RawHttpHeaders & ListMultivariateModelsDefaultHeaders;
}

export declare type ListMultivariateModelsParameters = ListMultivariateModelsQueryParam & RequestParameters;

export declare interface ListMultivariateModelsQueryParam {
    queryParameters?: ListMultivariateModelsQueryParamProperties;
}

export declare interface ListMultivariateModelsQueryParamProperties {
    /** Skip indicates how many models will be skipped. */
    skip?: number;
    /** Top indicates how many models will be fetched. */
    top?: number;
}

/**
 * Training result of a model including its status, errors and diagnostics
 * information.
 */
export declare interface ModelInfo {
    /**
     * Source link to the input data to indicate an accessible Azure storage Uri,
     * either pointed to an Azure blob storage folder, or pointed to a CSV file in
     * Azure blob storage based on you data schema selection.
     */
    dataSource: string;
    /**
     * Data schema of input data source: OneTable or MultiTable. The default
     * DataSchema is OneTable.
     *
     * Possible values: OneTable, MultiTable
     */
    dataSchema?: string;
    /**
     * A required field, indicating the start time of training data, which should be
     * date-time of ISO 8601 format.
     */
    startTime: Date | string;
    /**
     * A required field, indicating the end time of training data, which should be
     * date-time of ISO 8601 format.
     */
    endTime: Date | string;
    /**
     * An optional field. The display name of the model whose maximum length is 24
     * characters.
     */
    displayName?: string;
    /**
     * An optional field, indicating how many previous timestamps will be used to
     * detect whether the timestamp is anomaly or not.
     */
    slidingWindow?: number;
    /** An optional field, indicating the manner to align multiple variables. */
    alignPolicy?: AlignPolicy;
    /** Model status. One of CREATED, RUNNING, READY, and FAILED. */
    status?: "CREATED" | "RUNNING" | "READY" | "FAILED";
    /** Diagnostics information to help inspect the states of model or variable. */
    diagnosticsInfo?: DiagnosticsInfo;
}

/**
 * Training result of a model including its status, errors and diagnostics
 * information.
 */
export declare interface ModelInfoOutput {
    /**
     * Source link to the input data to indicate an accessible Azure storage Uri,
     * either pointed to an Azure blob storage folder, or pointed to a CSV file in
     * Azure blob storage based on you data schema selection.
     */
    dataSource: string;
    /**
     * Data schema of input data source: OneTable or MultiTable. The default
     * DataSchema is OneTable.
     *
     * Possible values: OneTable, MultiTable
     */
    dataSchema?: string;
    /**
     * A required field, indicating the start time of training data, which should be
     * date-time of ISO 8601 format.
     */
    startTime: string;
    /**
     * A required field, indicating the end time of training data, which should be
     * date-time of ISO 8601 format.
     */
    endTime: string;
    /**
     * An optional field. The display name of the model whose maximum length is 24
     * characters.
     */
    displayName?: string;
    /**
     * An optional field, indicating how many previous timestamps will be used to
     * detect whether the timestamp is anomaly or not.
     */
    slidingWindow?: number;
    /** An optional field, indicating the manner to align multiple variables. */
    alignPolicy?: AlignPolicyOutput;
    /** Model status. One of CREATED, RUNNING, READY, and FAILED. */
    status?: "CREATED" | "RUNNING" | "READY" | "FAILED";
    /** Error messages when failed to create a model. */
    readonly errors?: Array<ErrorResponseOutput>;
    /** Diagnostics information to help inspect the states of model or variable. */
    diagnosticsInfo?: DiagnosticsInfoOutput;
}

/** Response of listing models. */
export declare interface ModelListOutput {
    /** List of models. */
    models: Array<AnomalyDetectionModelOutput>;
    /** Number of trained multivariate models. */
    currentCount: number;
    /** Maximum number of models that can be trained for this Anomaly Detector resource. */
    maxCount: number;
    /** The link to fetch more models. */
    nextLink?: string;
}

/** Model status. */
export declare interface ModelState {
    /**
     * This indicates the number of passes of the entire training dataset the
     * algorithm has completed.
     */
    epochIds?: number[];
    /**
     * List of metrics used to assess how the model fits the training data for each
     * epoch.
     */
    trainLosses?: number[];
    /**
     * List of metrics used to assess how the model fits the validation set for each
     * epoch.
     */
    validationLosses?: number[];
    /** Latency for each epoch. */
    latenciesInSeconds?: number[];
}

/** Model status. */
export declare interface ModelStateOutput {
    /**
     * This indicates the number of passes of the entire training dataset the
     * algorithm has completed.
     */
    epochIds?: number[];
    /**
     * List of metrics used to assess how the model fits the training data for each
     * epoch.
     */
    trainLosses?: number[];
    /**
     * List of metrics used to assess how the model fits the validation set for each
     * epoch.
     */
    validationLosses?: number[];
    /** Latency for each epoch. */
    latenciesInSeconds?: number[];
}

/**
 * Detection request for batch inference. This is an asynchronous inference which
 * will need another API to get detection results.
 */
export declare interface MultivariateBatchDetectionOptions {
    /**
     * Source link to the input data to indicate an accessible Azure storage Uri,
     * either pointed to an Azure blob storage folder, or pointed to a CSV file in
     * Azure blob storage based on you data schema selection. The data schema should
     * be exactly the same with those used in the training phase.
     */
    dataSource: string;
    /**
     * An optional field, which is used to specify the number of top contributed
     * variables for one anomalous timestamp in the response. The default number is
     * 10.
     */
    topContributorCount: number;
    /**
     * A required field, indicating the start time of data for detection, which should
     * be date-time of ISO 8601 format.
     */
    startTime: Date | string;
    /**
     * A required field, indicating the end time of data for detection, which should
     * be date-time of ISO 8601 format.
     */
    endTime: Date | string;
}

/**
 * Detection request for batch inference. This is an asynchronous inference which
 * will need another API to get detection results.
 */
export declare interface MultivariateBatchDetectionOptionsOutput {
    /**
     * Source link to the input data to indicate an accessible Azure storage Uri,
     * either pointed to an Azure blob storage folder, or pointed to a CSV file in
     * Azure blob storage based on you data schema selection. The data schema should
     * be exactly the same with those used in the training phase.
     */
    dataSource: string;
    /**
     * An optional field, which is used to specify the number of top contributed
     * variables for one anomalous timestamp in the response. The default number is
     * 10.
     */
    topContributorCount: number;
    /**
     * A required field, indicating the start time of data for detection, which should
     * be date-time of ISO 8601 format.
     */
    startTime: string;
    /**
     * A required field, indicating the end time of data for detection, which should
     * be date-time of ISO 8601 format.
     */
    endTime: string;
}

/** Multivariate anomaly detection status. */
export declare interface MultivariateBatchDetectionResultSummaryOutput {
    /** Status of detection results. One of CREATED, RUNNING, READY, and FAILED. */
    status: "CREATED" | "RUNNING" | "READY" | "FAILED";
    /** Error message when detection is failed. */
    errors?: Array<ErrorResponseOutput>;
    /** Variable Status. */
    variableStates?: Array<VariableStateOutput>;
    /**
     * Detection request for batch inference. This is an asynchronous inference which
     * will need another API to get detection results.
     */
    setupInfo: MultivariateBatchDetectionOptionsOutput;
}

/** Detection results for the given resultId. */
export declare interface MultivariateDetectionResultOutput {
    /** Result identifier, which is used to fetch the results of an inference call. */
    resultId: string;
    /** Multivariate anomaly detection status. */
    summary: MultivariateBatchDetectionResultSummaryOutput;
    /** Detection result for each timestamp. */
    results: Array<AnomalyStateOutput>;
}

/** Request of last detection. */
export declare interface MultivariateLastDetectionOptions {
    /**
     * This contains the inference data, including the name, timestamps(ISO 8601) and
     * values of variables.
     */
    variables: Array<VariableValues>;
    /**
     * An optional field, which is used to specify the number of top contributed
     * variables for one anomalous timestamp in the response. The default number is
     * 10.
     */
    topContributorCount: number;
}

/** Results of last detection. */
export declare interface MultivariateLastDetectionResultOutput {
    /** Variable Status. */
    variableStates?: Array<VariableStateOutput>;
    /** Anomaly status and information. */
    results?: Array<AnomalyStateOutput>;
}

/**
 * Helper to paginate results from an initial response that follows the specification of Autorest `x-ms-pageable` extension
 * @param client - Client to use for sending the next page requests
 * @param initialResponse - Initial response containing the nextLink and current page of elements
 * @param customGetPage - Optional - Function to define how to extract the page and next link to be used to paginate the results
 * @returns - PagedAsyncIterableIterator to iterate the elements
 */
export declare function paginate<TResponse extends PathUncheckedResponse>(client: Client, initialResponse: TResponse, options?: PagingOptions<TResponse>): PagedAsyncIterableIterator<PaginateReturn<TResponse>>;

/**
 * Helper type to infer the Type of the paged elements from the response type
 * This type is generated based on the swagger information for x-ms-pageable
 * specifically on the itemName property which indicates the property of the response
 * where the page items are found. The default value is `value`.
 * This type will allow us to provide strongly typed Iterator based on the response we get as second parameter
 */
export declare type PaginateReturn<TResult> = TResult extends {
    body: {
        value?: infer TPage;
    };
} | {
    body: {
        models?: infer TPage;
    };
} ? GetArrayType<TPage> : Array<unknown>;

/**
 * Options for the paging helper
 */
export declare interface PagingOptions<TResponse> {
    /**
     * Custom function to extract pagination details for crating the PagedAsyncIterableIterator
     */
    customGetPage?: GetPage<PaginateReturn<TResponse>[]>;
}

export declare interface Routes {
    /** Resource for '/timeseries/entire/detect' has methods for the following verbs: post */
    (path: "/timeseries/entire/detect"): DetectUnivariateEntireSeries;
    /** Resource for '/timeseries/last/detect' has methods for the following verbs: post */
    (path: "/timeseries/last/detect"): DetectUnivariateLastPoint;
    /** Resource for '/timeseries/changepoint/detect' has methods for the following verbs: post */
    (path: "/timeseries/changepoint/detect"): DetectUnivariateChangePoint;
    /** Resource for '/multivariate/detect-batch/\{resultId\}' has methods for the following verbs: get */
    (path: "/multivariate/detect-batch/{resultId}", resultId: string): GetMultivariateBatchDetectionResult;
    /** Resource for '/multivariate/models' has methods for the following verbs: post, get */
    (path: "/multivariate/models"): TrainMultivariateModel;
    /** Resource for '/multivariate/models/\{modelId\}' has methods for the following verbs: delete, get */
    (path: "/multivariate/models/{modelId}", modelId: string): DeleteMultivariateModel;
    /** Resource for '/multivariate/models/\{modelId\}:detect-batch' has methods for the following verbs: post */
    (path: "/multivariate/models/{modelId}:detect-batch", modelId: string): DetectMultivariateBatchAnomaly;
    /** Resource for '/multivariate/models/\{modelId\}:detect-last' has methods for the following verbs: post */
    (path: "/multivariate/models/{modelId}:detect-last", modelId: string): DetectMultivariateLastAnomaly;
}

/** The definition of input timeseries points. */
export declare interface TimeSeriesPoint {
    /** Optional argument, timestamp of a data point (ISO8601 format). */
    timestamp?: Date | string;
    /** The measurement of that point, should be float. */
    value: number;
}

export declare interface TrainMultivariateModel {
    /**
     * Create and train a multivariate anomaly detection model. The request must
     * include a source parameter to indicate an externally accessible Azure blob
     * storage URI.There are two types of data input: An URI pointed to an Azure blob
     * storage folder which contains multiple CSV files, and each CSV file contains
     * two columns, timestamp and variable. Another type of input is an URI pointed to
     * a CSV file in Azure blob storage, which contains all the variables and a
     * timestamp column.
     */
    post(options: TrainMultivariateModelParameters): StreamableMethod<TrainMultivariateModel201Response | TrainMultivariateModelDefaultResponse>;
    /** List models of a resource. */
    get(options?: ListMultivariateModelsParameters): StreamableMethod<ListMultivariateModels200Response | ListMultivariateModelsDefaultResponse>;
}

export declare interface TrainMultivariateModel201Headers {
    /** Location and ID of the model. */
    location: string;
}

/** The request has succeeded and a new resource has been created as a result. */
export declare interface TrainMultivariateModel201Response extends HttpResponse {
    status: "201";
    body: AnomalyDetectionModelOutput;
    headers: RawHttpHeaders & TrainMultivariateModel201Headers;
}

export declare interface TrainMultivariateModelBodyParam {
    /** Model information. */
    body: ModelInfo;
}

export declare interface TrainMultivariateModelDefaultHeaders {
    /** Error code. */
    "x-ms-error-code"?: string;
}

export declare interface TrainMultivariateModelDefaultResponse extends HttpResponse {
    status: string;
    body: ErrorResponseOutput;
    headers: RawHttpHeaders & TrainMultivariateModelDefaultHeaders;
}

export declare type TrainMultivariateModelParameters = TrainMultivariateModelBodyParam & RequestParameters;

/** The request of change point detection. */
export declare interface UnivariateChangePointDetectionOptions {
    /**
     * Time series data points. Points should be sorted by timestamp in ascending
     * order to match the change point detection result.
     */
    series: Array<TimeSeriesPoint>;
    /**
     * Can only be one of yearly, monthly, weekly, daily, hourly, minutely or
     * secondly. Granularity is used for verify whether input series is valid.
     */
    granularity: "yearly" | "monthly" | "weekly" | "daily" | "hourly" | "minutely" | "secondly" | "microsecond" | "none";
    /**
     * Custom Interval is used to set non-standard time interval, for example, if the
     * series is 5 minutes, request can be set as {"granularity":"minutely",
     * "customInterval":5}.
     */
    customInterval?: number;
    /**
     * Optional argument, periodic value of a time series. If the value is null or
     * does not present, the API will determine the period automatically.
     */
    period?: number;
    /**
     * Optional argument, advanced model parameter, a default stableTrendWindow will
     * be used in detection.
     */
    stableTrendWindow?: number;
    /**
     * Optional argument, advanced model parameter, between 0.0-1.0, the lower the
     * value is, the larger the trend error will be which means less change point will
     * be accepted.
     */
    threshold?: number;
}

/** The response of change point detection. */
export declare interface UnivariateChangePointDetectionResultOutput {
    /**
     * Frequency extracted from the series, zero means no recurrent pattern has been
     * found.
     */
    readonly period?: number;
    /**
     * isChangePoint contains change point properties for each input point. True means
     * an anomaly either negative or positive has been detected. The index of the
     * array is consistent with the input series.
     */
    isChangePoint?: boolean[];
    /** the change point confidence of each point */
    confidenceScores?: number[];
}

/** The request of entire or last anomaly detection. */
export declare interface UnivariateDetectionOptions {
    /**
     * Time series data points. Points should be sorted by timestamp in ascending
     * order to match the anomaly detection result. If the data is not sorted
     * correctly or there is duplicated timestamp, the API will not work. In such
     * case, an error message will be returned.
     */
    series: Array<TimeSeriesPoint>;
    /**
     * Optional argument, can be one of yearly, monthly, weekly, daily, hourly,
     * minutely, secondly, microsecond or none. If granularity is not present, it will
     * be none by default. If granularity is none, the timestamp property in time
     * series point can be absent.
     */
    granularity?: "yearly" | "monthly" | "weekly" | "daily" | "hourly" | "minutely" | "secondly" | "microsecond" | "none";
    /**
     * Custom Interval is used to set non-standard time interval, for example, if the
     * series is 5 minutes, request can be set as {"granularity":"minutely",
     * "customInterval":5}.
     */
    customInterval?: number;
    /**
     * Optional argument, periodic value of a time series. If the value is null or
     * does not present, the API will determine the period automatically.
     */
    period?: number;
    /** Optional argument, advanced model parameter, max anomaly ratio in a time series. */
    maxAnomalyRatio?: number;
    /**
     * Optional argument, advanced model parameter, between 0-99, the lower the value
     * is, the larger the margin value will be which means less anomalies will be
     * accepted.
     */
    sensitivity?: number;
    /**
     * Used to specify how to deal with missing values in the input series, it's used
     * when granularity is not "none".
     *
     * Possible values: auto, previous, linear, fixed, zero, notFill
     */
    imputeMode?: string;
    /**
     * Used to specify the value to fill, it's used when granularity is not "none"
     * and imputeMode is "fixed".
     */
    imputeFixedValue?: number;
}

/** The response of entire anomaly detection. */
export declare interface UnivariateEntireDetectionResultOutput {
    /**
     * Frequency extracted from the series, zero means no recurrent pattern has been
     * found.
     */
    period: number;
    /**
     * ExpectedValues contain expected value for each input point. The index of the
     * array is consistent with the input series.
     */
    expectedValues: number[];
    /**
     * UpperMargins contain upper margin of each input point. UpperMargin is used to
     * calculate upperBoundary, which equals to expectedValue + (100 -
     * marginScale)*upperMargin. Anomalies in response can be filtered by
     * upperBoundary and lowerBoundary. By adjusting marginScale value, less
     * significant anomalies can be filtered in client side. The index of the array is
     * consistent with the input series.
     */
    upperMargins: number[];
    /**
     * LowerMargins contain lower margin of each input point. LowerMargin is used to
     * calculate lowerBoundary, which equals to expectedValue - (100 -
     * marginScale)*lowerMargin. Points between the boundary can be marked as normal
     * ones in client side. The index of the array is consistent with the input
     * series.
     */
    lowerMargins: number[];
    /**
     * IsAnomaly contains anomaly properties for each input point. True means an
     * anomaly either negative or positive has been detected. The index of the array
     * is consistent with the input series.
     */
    isAnomaly: boolean[];
    /**
     * IsNegativeAnomaly contains anomaly status in negative direction for each input
     * point. True means a negative anomaly has been detected. A negative anomaly
     * means the point is detected as an anomaly and its real value is smaller than
     * the expected one. The index of the array is consistent with the input series.
     */
    isNegativeAnomaly: boolean[];
    /**
     * IsPositiveAnomaly contain anomaly status in positive direction for each input
     * point. True means a positive anomaly has been detected. A positive anomaly
     * means the point is detected as an anomaly and its real value is larger than the
     * expected one. The index of the array is consistent with the input series.
     */
    isPositiveAnomaly: boolean[];
    /**
     * The severity score for each input point. The larger the value is, the more
     * sever the anomaly is. For normal points, the "severity" is always 0.
     */
    severity?: number[];
}

/** The response of last anomaly detection. */
export declare interface UnivariateLastDetectionResultOutput {
    /**
     * Frequency extracted from the series, zero means no recurrent pattern has been
     * found.
     */
    period: number;
    /** Suggested input series points needed for detecting the latest point. */
    suggestedWindow: number;
    /** Expected value of the latest point. */
    expectedValue: number;
    /**
     * Upper margin of the latest point. UpperMargin is used to calculate
     * upperBoundary, which equals to expectedValue + (100 - marginScale)*upperMargin.
     * If the value of latest point is between upperBoundary and lowerBoundary, it
     * should be treated as normal value. By adjusting marginScale value, anomaly
     * status of latest point can be changed.
     */
    upperMargin: number;
    /**
     * Lower margin of the latest point. LowerMargin is used to calculate
     * lowerBoundary, which equals to expectedValue - (100 - marginScale)*lowerMargin.
     *
     */
    lowerMargin: number;
    /**
     * Anomaly status of the latest point, true means the latest point is an anomaly
     * either in negative direction or positive direction.
     */
    isAnomaly: boolean;
    /**
     * Anomaly status in negative direction of the latest point. True means the latest
     * point is an anomaly and its real value is smaller than the expected one.
     */
    isNegativeAnomaly: boolean;
    /**
     * Anomaly status in positive direction of the latest point. True means the latest
     * point is an anomaly and its real value is larger than the expected one.
     */
    isPositiveAnomaly: boolean;
    /**
     * The severity score for the last input point. The larger the value is, the more
     * sever the anomaly is. For normal points, the "severity" is always 0.
     */
    severity?: number;
}

/** Variable Status. */
export declare interface VariableState {
    /** Variable name in variable states. */
    variable?: string;
    /** Proportion of missing values that need to be filled by fillNAMethod. */
    filledNARatio?: number;
    /** Number of effective data points before applying fillNAMethod. */
    effectiveCount?: number;
    /** First valid timestamp with value of input data. */
    firstTimestamp?: Date | string;
    /** Last valid timestamp with value of input data. */
    lastTimestamp?: Date | string;
}

/** Variable Status. */
export declare interface VariableStateOutput {
    /** Variable name in variable states. */
    variable?: string;
    /** Proportion of missing values that need to be filled by fillNAMethod. */
    filledNARatio?: number;
    /** Number of effective data points before applying fillNAMethod. */
    effectiveCount?: number;
    /** First valid timestamp with value of input data. */
    firstTimestamp?: string;
    /** Last valid timestamp with value of input data. */
    lastTimestamp?: string;
}

/** Variable values. */
export declare interface VariableValues {
    /** Variable name of last detection request. */
    variable: string;
    /** Timestamps of last detection request */
    timestamps: string[];
    /** Values of variables. */
    values: number[];
}

export { }
